/*======================================================================

    PCMCIA device status checker

    Written by David Hinds, dhinds@allegro.stanford.edu
    
======================================================================*/

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/ioctl.h>

#include "cs_types.h"
#include "cs.h"
#include "cistpl.h"
#include "ds.h"

static const char *version =
    "status.c v$Revision: 1.3 $: $Date: 1994/06/21 07:22:53 $ (David Hinds)\n";

/*====================================================================*/

void print_config(config_t *config)
{
    printf("  Vcc = %.1f, Vpp1 = %.1f, Vpp2 = %.1f\n",
	   config->Vcc/10.0, config->Vpp1/10.0, config->Vpp2/10.0);
    printf("  Card type is %s\n", (config->IntType == INT_MEMORY)
	   ? "memory-only" : "memory and I/O");
    printf("  Config register base = %#06lx\n",
	   (long)config->ConfigBase);
    
    if (config->CardValues) {
	printf("  ");
	if (config->CardValues & CV_OPTION_VALUE)
	    printf("  Option = %#04x", config->Option);
	if (config->CardValues & CV_STATUS_VALUE)
	    printf("  Status = %#04x", config->Status);
	if (config->CardValues & CV_PIN_REPLACEMENT)
	    printf("  Pin = %#04x", config->Pin);
	if (config->CardValues & CV_COPY_VALUE)
	    printf("  Copy = %#04x", config->Copy);
	printf("\n");
    }

    if (config->AssignedIRQ != 0) {
	printf("  IRQ %ld is ", config->AssignedIRQ);
	switch (config->IRQAttributes & IRQ_TYPE) {
	case IRQ_TYPE_EXCLUSIVE:
	    printf("exclusive"); break;
	case IRQ_TYPE_TIME:
	    printf("time-multiplexed"); break;
	case IRQ_TYPE_DYNAMIC_SHARING:
	    printf("dynamic shared"); break;
	}
	if (config->IRQAttributes & IRQ_PULSE_ALLOCATED)
	    printf(", pulse mode");
	else
	    printf(", level mode");
	printf(", %s\n", (config->Attributes & CONF_ENABLE_IRQ) ?
	       "enabled" : "disabled");
    }
    
    if (config->NumPorts1 > 0) {
	printf("  I/O window 1: %#06x to %#06lx", config->BasePort1,
	       config->BasePort1 + config->NumPorts1 - 1);
	if (config->Attributes1 & IO_SHARED)
	    printf(", shared");
	if (config->Attributes1 & IO_FORCE_ALIAS_ACCESS)
	    printf(", force alias");
	if (config->Attributes1 & IO_DATA_PATH_WIDTH)
	    printf(", 16 bit\n");
	else
	    printf(", 8 bit\n");
    }
    if (config->NumPorts2 > 0) {
	printf("  I/O window 2: %#06x to %#06lx", config->BasePort2,
	       config->BasePort2 + config->NumPorts2 - 1);
	if (config->Attributes1 & IO_SHARED)
	    printf(", shared");
	if (config->Attributes1 & IO_FORCE_ALIAS_ACCESS)
	    printf(", force alias");
	if (config->Attributes1 & IO_DATA_PATH_WIDTH)
	    printf(", 16 bit\n");
	else
	    printf(", 8 bit\n");
    }
} /* print_config */

/*====================================================================*/

#define MAX_SOCKS 8

void main(int argc, char *argv[])
{
    int i, fd, ret;
    char fn[20];
    config_t config;

    if ((argc > 1) && (strcmp(argv[1], "-v") == 0))
	printf("%s", version);
    
    for (i = 0; i < MAX_SOCKS; i++) {
	sprintf(fn, "/dev/pc%d", i);
	fd = open(fn, O_RDONLY);
	if (fd < 0) break;
	ret = ioctl(fd, DS_GET_CONFIGURATION_INFO, &config);
	if (ret == 0) {
	    printf("Socket %d:\n", i);
	    print_config(&config);
	}
    }
}
