/*
 * cs.h v$Revision: 1.9 $: $Date: 1994/06/21 07:16:12 $ (David Hinds)
 */

#ifndef _LINUX_CS_H
#define _LINUX_CS_H

/* For AccessConfigurationRegister */
typedef struct conf_reg_t {
    socket_t Socket;
    long Action;
    off_t Offset;
    u_long Value;
} conf_reg_t;

/* Actions */
#define CS_READ		1
#define CS_WRITE	2

/* for AdjustResourceInfo */
typedef struct adjust_t {
    int Action;
    int Resource;
    u_long Attributes;
    union {
	struct memory {
	    caddr_t Base;
	    int Size;
	} memory;
	struct io {
	    ioaddr_t BasePort;
	    int NumPorts;
	    int IOAddrLines;
	} io;
	struct irq {
	    int IRQ;
	} irq;
    } resource;
} adjust_t;

/* Action field */
#define REMOVE_MANAGED_RESOURCE		1
#define ADD_MANAGED_RESOURCE		2
#define GET_FIRST_MANAGED_RESOURCE	3
#define GET_NEXT_MANAGED_RESOURCE	4
/* Resource field */
#define RES_MEMORY_RANGE		1
#define RES_IO_RANGE			2
#define RES_IRQ				3
/* Attribute field */
#define RES_IRQ_TYPE			0x03
#define RES_IRQ_TYPE_EXCLUSIVE		0
#define RES_IRQ_TYPE_TIME		1
#define RES_IRQ_TYPE_DYNAMIC		2
#define RES_IRQ_CSC			0x04
#define RES_SHARED			0x08
#define RES_RESERVED			0x10
#define RES_ALLOCATED			0x20
#define RES_REMOVED			0x40

typedef struct servinfo_t {
    long InfoLen;
    char Signature[2];
    long Count;
    long Revision;
    long CSLevel;
    char *VendorString;
} servinfo_t;

typedef struct event_callback_args_t {
    client_handle_t client_handle;
    void *info;
    void *mtdrequest;
    void *buffer;
    void *misc;
    void *client_data;
} event_callback_args_t;

#ifdef __KERNEL__

#include <linux/ioport.h>

#define CLIENT_MAGIC 0x51E6
typedef struct client_t {
    u_long client_magic;
    dev_info_t *dev_info;
    u_long Attributes;
    u_long state;
    u_long EventMask;
    int (*event_handler)(event_t event, int priority,
			 event_callback_args_t *);
    event_callback_args_t event_callback_args;
    socket_t Socket;
    resource_entry_t io_1, io_2;
    struct client_t *next;
} client_t;

/* Flags in client state */
#define CLIENT_CONFIG_LOCKED	0x01
#define CLIENT_IRQ_REQ		0x02
#define CLIENT_IO_REQ		0x04
#define CLIENT_UNBOUND		0x08
#define CLIENT_STALE		0x10

#endif /* __KERNEL__ */

/* for GetConfigurationInfo */
typedef struct config_t {
    socket_t Socket;
    u_long Attributes;
    long Vcc, Vpp1, Vpp2;
    long IntType;
    caddr_t ConfigBase;
    u_char Status, Pin, Copy, Option;
    long Present;
    long FirstDevType, FuncCode;
    u_long SysInitMask;
    long ManufCode, ManufInfo;
    long CardValues;
    long AssignedIRQ;
    u_long IRQAttributes;
    ioaddr_t BasePort1;
    long NumPorts1;
    u_long Attributes1;
    ioaddr_t BasePort2;
    long NumPorts2;
    u_long Attributes2;
    long IOAddrLines;
} config_t;

/* For CardValues field */
#define CV_OPTION_VALUE		0x01
#define CV_STATUS_VALUE		0x02
#define CV_PIN_REPLACEMENT	0x04
#define CV_COPY_VALUE		0x08

/* For GetFirst/NextClient */
typedef struct client_req_t {
    socket_t Socket;
    u_long Attributes;
} client_req_t;

#define CLIENT_THIS_SOCKET	0x01

/* For RegisterClient */
typedef struct client_reg_t {
    dev_info_t *dev_info;
    socket_t Socket;
    u_long Attributes;
    u_long EventMask;
    int (*event_handler)(event_t event, int priority,
			 event_callback_args_t *);
    event_callback_args_t event_callback_args;
    u_long Version;
} client_reg_t;

/* For RequestConfiguration */
typedef struct config_req_t {
    socket_t Socket;
    u_long Attributes;
    long Vcc, Vpp1, Vpp2;
    long IntType;
    caddr_t ConfigBase;
    long Status, Pin, Copy;
    long ConfigIndex;
    long Present;
} config_req_t;

/* Attributes for RequestConfiguration */
#define CONF_ENABLE_IRQ		0x01
/* IntType field */
#define INT_MEMORY		0x01
#define INT_MEMORY_AND_IO	0x02

/* For RequestIO and ReleaseIO */
typedef struct io_req_t {
    socket_t Socket;
    ioaddr_t BasePort1;
    long NumPorts1;
    u_long Attributes1;
    ioaddr_t BasePort2;
    long NumPorts2;
    u_long Attributes2;
    long IOAddrLines;
} io_req_t;

/* Attributes for RequestIO and ReleaseIO */
#define IO_SHARED		0x01
#define IO_FIRST_SHARED		0x02
#define IO_FORCE_ALIAS_ACCESS	0x04
#define IO_DATA_PATH_WIDTH	0x08

/* For RequestIRQ and ReleaseIRQ */
typedef struct irq_req_t {
    socket_t Socket;
    u_long Attributes;
    long AssignedIRQ;
    long IRQInfo1, IRQInfo2;
} irq_req_t;

/* Attributes for RequestIRQ and ReleaseIRQ */
#define IRQ_TYPE		0x03
#define IRQ_TYPE_EXCLUSIVE	0x00
#define IRQ_TYPE_TIME		0x01
#define IRQ_TYPE_DYNAMIC_SHARING 0x02
#define IRQ_FORCED_PULSE	0x04
#define IRQ_FIRST_SHARED	0x08
#define IRQ_PULSE_ALLOCATED	0x10

/* Bits in IRQInfo1 field */
#define IRQ_MASK		0x0F
#define IRQ_NMI_ID		0x01
#define IRQ_IOCK_ID		0x02
#define IRQ_BERR_ID		0x04
#define IRQ_VEND_ID		0x08
#define IRQ_INFO2_VALID		0x10
#define IRQ_LEVEL_ID		0x20
#define IRQ_PULSE_ID		0x40
#define IRQ_SHARE_ID		0x80

typedef struct eventmask_t {
    u_long Attributes;
    u_long EventMask;
    socket_t Socket;
} eventmask_t;
#define CONF_EVENT_MASK_VALID	0x01

/* Configuration registers present */
#define PRESENT_OPTION		0x01
#define PRESENT_STATUS		0x02
#define PRESENT_PIN_REPLACE	0x04
#define PRESENT_COPY		0x08

/* Attributes for Request/GetConfiguration */
#define CONF_ENABLE_IRQ		0x01
#define EXCLUSIVE_USE		0x02
#define VALID_CLIENT		0x04

/* Attributes for RegisterClient */
#define INFO_DRIVER_SERVICES	0x01
#define INFO_IO_CLIENT		0x02
#define INFO_MTD_CLIENT		0x04
#define INFO_MEM_CLIENT		0x08
#define MAX_NUM_CLIENTS		3

#define INFO_CARD_SHARE		0x10
#define INFO_CARD_EXCL		0x20

typedef struct tuple_t {
    socket_t Socket;
    u_long Attributes;
    cisdata_t DesiredTuple;
    u_long Flags;
    cisdata_t TupleCode;
    long TupleLink;
    cisdata_t TupleOffset;
    cisdata_t TupleDataMax;
    cisdata_t TupleDataLen;
    cisdata_t *TupleData;
} tuple_t;

/* Special cisdata_t value */
#define RETURN_FIRST_TUPLE	0xf0

/* Attributes for tuple calls */
#define TUPLE_RETURN_LINK	0x01

typedef struct status_t {
    socket_t Socket;
    long CardState;
    long SocketState;
} status_t;

/* Event mask bits */
#define EVENT_WRITE_PROTECT	0x01
#define EVENT_CARD_LOCK		0x02
#define EVENT_EJECTION		0x04
#define EVENT_INSERTION		0x08
#define EVENT_BATTERY_DEAD	0x10
#define EVENT_BATTERY_LOW	0x20
#define EVENT_READY_CHANGE	0x40
#define EVENT_CARD_DETECT	0x80
#define EVENT_PM_CHANGE		0x100
#define EVENT_RESET		0x200

/* Events */
#define CS_EVENT_PRI_LOW	0
#define CS_EVENT_PRI_HIGH	1

#define CS_EVENT_REGISTRATION_COMPLETE	1
#define CS_EVENT_CARD_INSERTION		2
#define CS_EVENT_CARD_REMOVAL		3
#define CS_EVENT_BATTERY_DEAD		4
#define CS_EVENT_BATTERY_LOW		5
#define CS_EVENT_READY_CHANGE		6
#define CS_EVENT_RESET_REQUEST		7
#define CS_EVENT_RESET_PHYSICAL		8
#define CS_EVENT_CARD_RESET		9
#define CS_EVENT_RESET_COMPLETE		10

/* Return codes */
#define CS_SUCCESS		0x00
#define CS_BAD_ADAPTER		0x01
#define CS_BAD_ATTRIBUTE	0x02
#define CS_BAD_BASE		0x03
#define CS_BAD_EDC		0x04
#define CS_BAD_IRQ		0x06
#define CS_BAD_OFFSET		0x07
#define CS_BAD_PAGE		0x08
#define CS_READ_FAILURE		0x09
#define CS_BAD_SIZE		0x0a
#define CS_BAD_SOCKET		0x0b
#define CS_BAD_TYPE		0x0d
#define CS_BAD_VCC		0x0e
#define CS_BAD_VPP		0x0f
#define CS_BAD_WINDOW		0x11
#define CS_WRITE_FAILURE	0x12
#define CS_NO_CARD		0x14
#define CS_UNSUPPORTED_FUNCTION	0x15
#define CS_UNSUPPORTED_MODE	0x16
#define CS_BAD_SPEED		0x17
#define CS_BUSY			0x18
#define CS_GENERAL_FAILURE	0x19
#define CS_WRITE_PROTECTED	0x1a
#define CS_BAD_ARG_LENGTH	0x1b
#define CS_BAD_ARGS		0x1c
#define CS_CONFIGURATION_LOCKED	0x1d
#define CS_IN_USE		0x1e
#define CS_NO_MORE_ITEMS	0x1f
#define CS_OUT_OF_RESOURCE	0x20
#define CS_BAD_HANDLE		0x21

#define CS_BAD_TUPLE		0x40

#ifdef __KERNEL__

enum service {
    AccessConfigurationRegister, AddSocketServices,
    AdjustResourceInfo, CheckEraseQueue, CloseMemory, CopyMemory,
    DeregisterClient, DeregisterEraseQueue, GetCardServicesInfo,
    GetClientInfo, GetConfigurationInfo, GetEventMask,
    GetFirstClient, GetFirstPartion, GetFirstRegion, GetFirstTuple,
    GetNextClient, GetNextPartition, GetNextRegion, GetNextTuple,
    GetStatus, GetTupleData, MapLogSocket, MapLogWindow, MapMemPage,
    MapPhySocket, MapPhyWindow, ModifyConfiguration, ModifyWindow,
    OpenWindow, ParseTuple, ReadMemory, RegisterClient,
    RegisterEraseQueue, RegisterMTD, RegisterTimer,
    ReleaseConfiguration, ReleaseExclusive, ReleaseIO, ReleaseIRQ,
    ReleaseSocketMask, ReleaseWindow, ReplaceSocketServices,
    RequestConfiguration, RequestExclusive, RequestIO, RequestIRQ,
    RequestSocketMask, RequestWindow, ResetCard, ReturnSSEntry,
    SetEventMask, SetRegion, ValidateCIS, VendorSpecific,
    WriteMemory
};

int CardServices(int func, ...);

typedef struct bind_req_t {
    socket_t Socket;
    dev_info_t *dev_info;
} bind_req_t;

int bind_device(bind_req_t *req);

#endif /* __KERNEL__ */

#endif /* _LINUX_CARD_SERVICES_H */
