/*
 * ds.h v$Revision: 1.9 $: $Date: 1994/06/21 07:26:19 $ (David Hinds)
 */

#ifndef _LINUX_DS_H
#define _LINUX_DS_H

#ifdef __KERNEL__

typedef struct dev_link_t {
    char dev_name[8];
    int state;
    struct timer_list config, release;
    client_handle_t handle;
    io_req_t io;
    irq_req_t irq;
    config_req_t conf;
    void *priv;
    struct dev_link_t *next;
} dev_link_t;

/* Flags for device state */
#define DEV_PRESENT		0x01
#define DEV_CONFIG		0x02
#define DEV_STALE_CONFIG	0x04	/* release on close */
#define DEV_STALE_LINK		0x08	/* detach on release */
#define DEV_CONFIG_PENDING	0x10

int register_pcmcia_driver(dev_info_t *dev_info,
			   dev_link_t *(*attach)(void),
			   void (*detach)(dev_link_t *));

int unregister_pcmcia_driver(dev_info_t *dev_info);

#endif /* __KERNEL__ */

typedef struct tuple_parse_t {
    tuple_t tuple;
    char data[255];
    cisparse_t parse;
} tuple_parse_t;

typedef struct bind_info_t {
    dev_info_t dev_info;
    struct dev_link_t *instance;
    char name[8];
    struct socket_bind_t *next;
} bind_info_t;

typedef union ds_ioctl_arg_t {
    servinfo_t servinfo;
    adjust_t adjust;
    config_t config;
    tuple_t tuple;
    tuple_parse_t tuple_parse;
    client_req_t client_req;
    bind_info_t bind_info;
} ds_ioctl_arg_t;

#define DS_EVENT_CARD_INSERTION 0x01
#define DS_EVENT_CARD_REMOVAL	0x02
#define DS_EVENT_RESET_COMPLETE	0x04

#define DS_GET_CARD_SERVICES_INFO	_IOR ('d', 1, servinfo_t)
#define DS_ADJUST_RESOURCE_INFO		_IOWR('d', 2, adjust_t)
#define DS_GET_CONFIGURATION_INFO	_IOR ('d', 3, config_t)
#define DS_GET_FIRST_TUPLE		_IOWR('d', 4, tuple_t)
#define DS_GET_NEXT_TUPLE		_IOWR('d', 5, tuple_t)
#define DS_GET_TUPLE_DATA		_IOWR('d', 6, tuple_parse_t)
#define DS_PARSE_TUPLE			_IOWR('d', 7, tuple_parse_t)
#define DS_RESET_CARD			_IOW ('d', 8, client_req_t)

#define DS_BIND_REQUEST			_IOWR('d',60, bind_info_t)
#define DS_GET_DEVICE_INFO		_IOWR('d',61, bind_info_t) 
#define DS_UNBIND_REQUEST		_IOW ('d',62, bind_info_t)

#endif /* _LINUX_DS_H */
