/*
 * pcmcia.h v$Revision: 1.1 $: $Date: 1994/06/21 07:58:21 $ (David Hinds)
 */

#ifndef _LINUX_PCMCIA_H
#define _LINUX_PCMCIA_H

#include <sys/ioctl.h>

#ifdef __KERNEL__

typedef struct pcmcia_ctrl_t {
    char *name;
    int sock;
    int flags;
    int csc_irq;
    int (*open)(struct pcmcia_ctrl_t *);
    void (*close)(struct pcmcia_ctrl_t *);
    int (*ioctl)(struct pcmcia_ctrl_t *, u_int, void *);
    struct pcmcia_ctrl_t *next;
} pcmcia_ctrl_t;

int register_pcmcia(pcmcia_ctrl_t *);
void unregister_pcmcia(pcmcia_ctrl_t *);

#endif /* KERNEL */

/* Definitions for card status flags */
#define PC_WRPROT	0x01
#define PC_CARDLOCK	0x02
#define PC_EJECTION	0x04
#define PC_INSERTION	0x08
#define PC_BATDEAD	0x10
#define PC_BATWARN	0x20
#define PC_READY	0x40
#define PC_DETECT	0x80
#define PC_POWERON	0x100
#define PC_GPI		0x200
#define PC_STSCHG	0x400

/* Various card configuration flags */
#define PC_PWR_OFF	0x00
#define PC_VCC_MASK	0x03
#define PC_VCC_3V	0x01
#define PC_VCC_5V	0x02
#define PC_VPP_MASK	0x0C
#define PC_VPP_5V	0x04
#define PC_VPP_12V	0x08
#define PC_PWR_AUTO	0x10
#define PC_IOCARD	0x20
#define PC_RESET	0x40

/* Flags for I/O port and memory windows */
#define MAP_ACTIVE	0x01
#define MAP_16BIT	0x02
#define MAP_0WS		0x04
#define MAP_CS16	0x08
#define MAP_WRPROT	0x10
#define MAP_ATTRIB	0x20

/* Definitions for I/O port window ioctl calls */
struct pcmcia_io_map {
  u_char map, flags, extra_ws;
  u_short start, stop;
  };

/* Definitions for memory window ioctl calls */
struct pcmcia_mem_map {
  u_char map, flags, extra_ws;
  u_long sys_start, sys_stop;
  u_long card_start;
  };

/* ioctl definitions for PCMCIA devices */
#define PCMCIA_GET_STATUS	_IOR ('p', 0, u_int)
#define PCMCIA_GET_FLAGS	_IOR ('p', 1, u_int)
#define PCMCIA_SET_FLAGS	_IOW ('p', 2, u_int)
#define PCMCIA_CSC_ON		_IOW ('p', 3, u_int)
#define PCMCIA_CSC_OFF		_IO  ('p', 4)
#define PCMCIA_CSC_POLL		_IOWR('p', 5, u_int)
#define PCMCIA_GET_IRQ		_IOR ('p', 6, u_int)
#define PCMCIA_SET_IRQ		_IOW ('p', 7, u_int)
#define PCMCIA_GET_IO_MAP	_IOWR('p', 8, struct pcmcia_io_map)
#define PCMCIA_SET_IO_MAP	_IOW ('p', 9, struct pcmcia_io_map)
#define PCMCIA_GET_MEM_MAP	_IOWR('p',10, struct pcmcia_mem_map)
#define PCMCIA_SET_MEM_MAP	_IOW ('p',11, struct pcmcia_mem_map)
#define PCMCIA_RESET		_IO  ('p',12)

#endif /* _LINUX_PCMCIA_H */
